<?php

/**
 * This code was generated by
 * \ / _    _  _|   _  _
 * | (_)\/(_)(_|\/| |(/_  v1.0.0
 * /       /
 */

namespace Twilio\Rest\Trusthub\V1\TrustProducts;

use Twilio\Exceptions\TwilioException;
use Twilio\ListResource;
use Twilio\Stream;
use Twilio\Values;
use Twilio\Version;

class TrustProductsEntityAssignmentsList extends ListResource {
    /**
     * Construct the TrustProductsEntityAssignmentsList
     *
     * @param Version $version Version that contains the resource
     * @param string $trustProductSid The unique string that identifies the
     *                                TrustProduct resource.
     */
    public function __construct(Version $version, string $trustProductSid) {
        parent::__construct($version);

        // Path Solution
        $this->solution = ['trustProductSid' => $trustProductSid, ];

        $this->uri = '/TrustProducts/' . \rawurlencode($trustProductSid) . '/EntityAssignments';
    }

    /**
     * Create the TrustProductsEntityAssignmentsInstance
     *
     * @param string $objectSid The sid of an object bag
     * @return TrustProductsEntityAssignmentsInstance Created
     *                                                TrustProductsEntityAssignmentsInstance
     * @throws TwilioException When an HTTP error occurs.
     */
    public function create(string $objectSid): TrustProductsEntityAssignmentsInstance {
        $data = Values::of(['ObjectSid' => $objectSid, ]);

        $payload = $this->version->create('POST', $this->uri, [], $data);

        return new TrustProductsEntityAssignmentsInstance(
            $this->version,
            $payload,
            $this->solution['trustProductSid']
        );
    }

    /**
     * Streams TrustProductsEntityAssignmentsInstance records from the API as a
     * generator stream.
     * This operation lazily loads records as efficiently as possible until the
     * limit
     * is reached.
     * The results are returned as a generator, so this operation is memory
     * efficient.
     *
     * @param int $limit Upper limit for the number of records to return. stream()
     *                   guarantees to never return more than limit.  Default is no
     *                   limit
     * @param mixed $pageSize Number of records to fetch per request, when not set
     *                        will use the default value of 50 records.  If no
     *                        page_size is defined but a limit is defined, stream()
     *                        will attempt to read the limit with the most
     *                        efficient page size, i.e. min(limit, 1000)
     * @return Stream stream of results
     */
    public function stream(int $limit = null, $pageSize = null): Stream {
        $limits = $this->version->readLimits($limit, $pageSize);

        $page = $this->page($limits['pageSize']);

        return $this->version->stream($page, $limits['limit'], $limits['pageLimit']);
    }

    /**
     * Reads TrustProductsEntityAssignmentsInstance records from the API as a list.
     * Unlike stream(), this operation is eager and will load `limit` records into
     * memory before returning.
     *
     * @param int $limit Upper limit for the number of records to return. read()
     *                   guarantees to never return more than limit.  Default is no
     *                   limit
     * @param mixed $pageSize Number of records to fetch per request, when not set
     *                        will use the default value of 50 records.  If no
     *                        page_size is defined but a limit is defined, read()
     *                        will attempt to read the limit with the most
     *                        efficient page size, i.e. min(limit, 1000)
     * @return TrustProductsEntityAssignmentsInstance[] Array of results
     */
    public function read(int $limit = null, $pageSize = null): array {
        return \iterator_to_array($this->stream($limit, $pageSize), false);
    }

    /**
     * Retrieve a single page of TrustProductsEntityAssignmentsInstance records
     * from the API.
     * Request is executed immediately
     *
     * @param mixed $pageSize Number of records to return, defaults to 50
     * @param string $pageToken PageToken provided by the API
     * @param mixed $pageNumber Page Number, this value is simply for client state
     * @return TrustProductsEntityAssignmentsPage Page of
     *                                            TrustProductsEntityAssignmentsInstance
     */
    public function page($pageSize = Values::NONE, string $pageToken = Values::NONE, $pageNumber = Values::NONE): TrustProductsEntityAssignmentsPage {
        $params = Values::of(['PageToken' => $pageToken, 'Page' => $pageNumber, 'PageSize' => $pageSize, ]);

        $response = $this->version->page('GET', $this->uri, $params);

        return new TrustProductsEntityAssignmentsPage($this->version, $response, $this->solution);
    }

    /**
     * Retrieve a specific page of TrustProductsEntityAssignmentsInstance records
     * from the API.
     * Request is executed immediately
     *
     * @param string $targetUrl API-generated URL for the requested results page
     * @return TrustProductsEntityAssignmentsPage Page of
     *                                            TrustProductsEntityAssignmentsInstance
     */
    public function getPage(string $targetUrl): TrustProductsEntityAssignmentsPage {
        $response = $this->version->getDomain()->getClient()->request(
            'GET',
            $targetUrl
        );

        return new TrustProductsEntityAssignmentsPage($this->version, $response, $this->solution);
    }

    /**
     * Constructs a TrustProductsEntityAssignmentsContext
     *
     * @param string $sid The unique string that identifies the resource
     */
    public function getContext(string $sid): TrustProductsEntityAssignmentsContext {
        return new TrustProductsEntityAssignmentsContext(
            $this->version,
            $this->solution['trustProductSid'],
            $sid
        );
    }

    /**
     * Provide a friendly representation
     *
     * @return string Machine friendly representation
     */
    public function __toString(): string {
        return '[Twilio.Trusthub.V1.TrustProductsEntityAssignmentsList]';
    }
}